package org.tidestudio.utils;

import weka.classifiers.Classifier;
import weka.core.Instances;

public final class TideFSUtils {

    public static void distributeInstancesByClassesEvenly(final Instances all) {
        int[] nInsPerClass = all.attributeStats(all.classIndex()).nominalCounts;

        int nClasses = nInsPerClass.length;

        double[] ratioPerClass = new double[nClasses];
        for (int i = 0; i < nClasses; i++) {
            ratioPerClass[i] = (double) nInsPerClass[i] / all.numInstances();
        }

        int[] currCount = new int[nInsPerClass.length];
        currCount[0] = 1;
        for (int i = 1; i < all.numInstances(); ++i) {
            if ((int) all.instance(0).classValue() != 0) {
                all.swap(0, i);
            } else {
                break;
            }
        }

        for (int i = 1; i < all.numInstances(); i++) {
            for (int j = 0; j < nClasses; j++) {
                if (((double) currCount[j] / (i + 1)) < ratioPerClass[j]) {
                    currCount[j]++;
                    for (int k = i + 1; k < all.numInstances(); k++) {
                        if ((int) all.instance(i).classValue() != j) {
                            all.swap(i, k);
                        } else {
                            break;
                        }
                    }
                    break;
                }
            }
        }
    }

    public static double errorRate(final Classifier classifier, final Instances test) throws Exception {
        double err = 0;
        int a, b;
        for (int i = 0; i < test.numInstances(); i++) {
            a = (int) classifier.classifyInstance(test.instance(i));
            b = (int) test.instance(i).classValue();
            if (a != b) {
                err += 1.0;
            }
        }
        return err / test.numInstances();
    }

    public static Instances instancesBySelectedFeatures(final Instances instances, final int[] features) {
        Instances newInstances = new Instances(instances);

        int tmp = features.length - 1;

        for (int i = instances.numAttributes() - 1; i >= 0; i--) {
            if ((i != features[(tmp > 0) ? tmp : 0]) && (i != instances.classIndex())) {
                newInstances.deleteAttributeAt(i);
            } else {
                tmp--;
            }
        }
        return newInstances;
    }
}
