package weka.attributeSelection;

import org.tidestudio.utils.TideFSUtils;

import java.util.BitSet;
import java.util.Random;

import weka.classifiers.AbstractClassifier;
import weka.classifiers.Classifier;
import weka.core.Instances;

public class Evaluator extends ASEvaluation implements SubsetEvaluator {

    private static final long serialVersionUID = 5275774502871902372L;

    private Classifier        m_classifier;
    private Instances         m_instances;
    private int               m_folds;
    private Random            m_rnd;

    public Evaluator(Classifier classifier, Instances instances, int folds, long randSeed) throws Exception {
        m_classifier = AbstractClassifier.makeCopy(classifier);

        m_instances = new Instances(instances);
        m_folds = folds;
        m_rnd = new Random(randSeed);
        if (m_folds > 1) {
            m_instances.randomize(m_rnd);
            m_instances.stratify(m_folds);
        }
    }

    @Override
    public double evaluateSubset(BitSet subset) throws Exception {

        Instances instances = new Instances(m_instances);
        Classifier c = AbstractClassifier.makeCopy(m_classifier);

        for (int i = instances.numAttributes() - 1; i >= 0; i--) {
            if (!subset.get(i) && (i != instances.classIndex())) {
                instances.deleteAttributeAt(i);
            }
        }

        if (instances.numAttributes() == 1) {
            return -Double.MAX_VALUE;
        }

        double err = 0;
        for (int n = 0; n < m_folds; n++) {
            Instances train;
            Instances test;

            if (m_folds > 1) {
                train = instances.trainCV(m_folds, n);
                test = instances.testCV(m_folds, n);
            } else {
                train = new Instances(instances);
                test = new Instances(instances);
            }

            c.buildClassifier(train);

            err += TideFSUtils.errorRate(c, test);
        }
        return -err / m_folds;
    }

    @Override
    public void buildEvaluator(Instances data) throws Exception {
        // TODO Auto-generated method stub

    }

}
